@@ -1,9 +1,9 @@
-from napari_generic_simulator import SIMulator_widget
+from napari_generic_simulator import SIMulator
 
 
 # make_napari_viewer is a pytest fixture that returns a napari viewer object
 # capsys is a pytest fixture that captures stdout and stderr output streams
-def test_widget(make_napari_viewer, capsys):
+def test_simulator_widget(make_napari_viewer, capsys):
 
     viewer = make_napari_viewer()
-    simulator_widget = SIMulator_widget(viewer)
\ No newline at end of file
+    simulator_widget = SIMulator(viewer)
\ No newline at end of file
